/* Fltkmchart.h */

/*
Copyright (c) 2015, The Fltkmchart Authors

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef FLTKMCHART_H
#define FLTKMCHART_H

#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include <functional>
#include <algorithm>
#include <vector>

#define INHERIT_FLWIDGET_CTOR(A, B) A(int X, int Y, int W, int H, const char *L = 0) : B(X, Y, W, H, L)

//

class Fltkmchart_Series {
public:
	std::vector<std::pair<double, double>> points;
	std::string legend;
	Fl_Color color;
};

//

class Fltkmchart : public Fl_Widget {

protected:
	std::function<std::string(double)>	xaxis_renderer,
										yaxis_renderer;
	std::function<unsigned int(double)> xtf,
										ytf;
	std::vector<Fltkmchart_Series>		series;
	Fl_Color							backgroundColor,
										axisColor;
	unsigned int						xaxis_padding = 35,
										yaxis_padding = 20;	
	double								x_min,
										y_min,
										x_max,
										y_max;
	Fl_Font								labelFontFace;
	Fl_Fontsize							labelFontSize;

public:

	INHERIT_FLWIDGET_CTOR(Fltkmchart, Fl_Widget),
		backgroundColor(FL_WHITE),
		axisColor(FL_BLACK)
	{
		xaxis_renderer = getNullAxisRenderer();
		yaxis_renderer = getNullAxisRenderer();
		setDrawFont(FL_HELVETICA, 10);
		clearSeries();
	}

	void setXAxisRenderer(std::function<std::string(double)> cb) {
		xaxis_renderer = cb;
	}

	void setYAxisRenderer(std::function<std::string(double)> cb) {
		yaxis_renderer = cb;
	}

	void setDrawFont(Fl_Font fontid, Fl_Fontsize size) {
		labelFontFace = fontid;
		labelFontSize = size;
	}

	std::function<std::string(double)> getNullAxisRenderer();

	void clearSeries();

	void addSeries(Fltkmchart_Series data);

	void drawLine_screenSpace(int x1, int y1, int x2, int y2, Fl_Color fc);

	void drawRect_screenSpace(int x, int y, int w, int h, Fl_Color fc);	

	void drawText_screenSpace(const char* text, int x, int y, Fl_Color fc);

	void drawLine_dataSpace(double x1, double y1, double x2, double y2, Fl_Color fc);

	void drawRect_dataSpace(double x1, double y1, double x2, double y2, Fl_Color fc);

	size_t getTextWidth(const char* text);

	void draw();

	void resize(int x, int y, int w, int h);

protected:
	std::function<unsigned int(double)> getTransformFunction(double min, double max, unsigned int range, unsigned int padding);

	void recalculateTransformFunctions();

	virtual void drawSeries() = 0;

};

//

class Fltkmchart_Line : public Fltkmchart {

public:
	INHERIT_FLWIDGET_CTOR(Fltkmchart_Line, Fltkmchart) { }

protected:
	void drawSeries();

};

//

class Fltkmchart_StaggeredLine : public Fltkmchart {

public:
	INHERIT_FLWIDGET_CTOR(Fltkmchart_StaggeredLine, Fltkmchart) { }

protected:
	void drawSeries();

};

//

class Fltkmchart_Bar : public Fltkmchart {

public:
	INHERIT_FLWIDGET_CTOR(Fltkmchart_Bar, Fltkmchart) { }

protected:
	void drawSeries();
};

//

#endif /* FLTKMCHART_H */
