#include "FuelRecord.h"
#include <FL\Fl.H>
#include <FL\fl_ask.H>
#include <regex>

std::string FuelRecord::serialise() {
	char buff[4096] = { 0 };
	sprintf_s(buff, "%s\n%1.2f\n%1.2f\n%s\n%s\n", date.c_str(), odo, fuel_added, filled ? "Y" : "N", comment.c_str());
	return buff;
}

std::string FuelRecord::validate() {

	// FIXME: assert monotonicity

	if (date.length() && date[date.length() - 1] == '\n') {
		date.pop_back();
	}

	if (date.length()) {
		std::regex date_reg("^(\\d{4})-(\\d{2})-(\\d{2})$");
		std::smatch match_parts;
		if (!std::regex_match(date, match_parts, date_reg)) {
			return "Date should be either blank or in YYYY-MM-DD format.";
		}

		struct tm timespec = { 0 };
		timespec.tm_year = atoi(match_parts[1].str().c_str()) - 1900;
		timespec.tm_mon = atoi(match_parts[2].str().c_str()) - 1;
		timespec.tm_mday = atoi(match_parts[3].str().c_str());
		date_time = mktime(&timespec);

	} else {
		date_time = 0; // value unknown

	}

	if (odo < 0.0) {
		return "Odometer reading can't be negative.";
	}

	if (fuel_added < 0.0 || fuel_added > 100.0) {
		return "Fuel added outside expected bounds 0..100L";
	}

	if (comment.length() && comment[comment.length() - 1] == '\n') {
		comment.pop_back();
	}

	if (comment.length() && comment[comment.length() - 1] == '\r') {
		comment.pop_back();
	}

	return ""; // no problem
}

bool FuelRecord::parse_failure(const char* warning) {
	fl_alert("WARNING: Failed to parse record: %s", warning);
	return false;
}

bool FuelRecord::deserialise_from(FILE* f) {
	char buff[4096] = { 0 };

	if (fgets(buff, sizeof(buff), f) == nullptr) return false;
	date = buff;

	if (fgets(buff, sizeof(buff), f) == nullptr) return false;
	odo = atof(buff);

	if (fgets(buff, sizeof(buff), f) == nullptr) return false;
	fuel_added = atof(buff);

	if (fgets(buff, sizeof(buff), f) == nullptr) return false;
	if (buff[0] == 'Y') {
		filled = true;
	} else if (buff[0] == 'N') {
		filled = false;
	} else {
		return parse_failure("Corrupt filled state");
	}

	if (fgets(buff, sizeof(buff), f) == nullptr) return false;
	comment = buff;

	auto error = validate();
	if (error.length()) {
		return parse_failure(error.c_str());
	}

	return true;
}
